package app.main;

import ryulib.Direction;
import ryulib.OnNotifyEventListener;
import ryulib.game.GameControlGroup;
import android.graphics.Point;

public class Blocks implements IBlocks {
	
	public Blocks(IBoard board) {
		super();
		
		_Board = board;
	}

	private IBoard _Board;
	
	private Block _Blank = null;
	private Block[][] _Blocks = null;
	private int _Width = 0;
	private int _Height = 0;

	public void setSize(int width, int height) {
		_Width = width;
		_Height = height;
		
		align();
	}

	@Override
	public void align() {
		_Blocks = new Block[_Width][_Height];
		
		((GameControlGroup) _Board).clearControls();
		
		for (int y=0; y<_Height; y++) {
			for (int x=0; x<_Width; x++) {
				// TODO
				_Blocks[x][y] = new Block(((GameControlGroup) _Board));
				_Blocks[x][y].setNo(x + _Width*y + 1);  
				_Blocks[x][y].setSize(_Board.getBlockSize());  
				_Blocks[x][y].setPosition(x, y);  
				_Blocks[x][y].setIsBlankBlock(false);
				_Blocks[x][y].setOnClick(_OnBlockClick);
			}	
		}
		
		_Blank = _Blocks[_Width-1][_Height-1];
		_Blank.setIsBlankBlock(true);
	}

	@Override
	public int getWidth() {
		return _Width;
	}

	@Override
	public int getHeight() {
		return _Height;
	}

	@Override
	public Block getBlock(Point point) {
		return _Blocks[point.x][point.y];
	}

	@Override
	public Direction getBlankDirection(Point point) {
		Direction result = Direction.NoWhere;
		
		if (point.x == _Blank.getPosition().x) {
			if (point.y > _Blank.getPosition().y) result = Direction.Up;
			else result = Direction.Down;
		} else if (point.y == _Blank.getPosition().y) {
			if (point.x > _Blank.getPosition().x) result = Direction.Left;
			else result = Direction.Right;
		}
		
		return result;
	}

	@Override
	public Point getBlankPosition() {
		return _Blank.getPosition();
	}
	
	private void swapBlocks(Block a, Block b) {
		_Blocks[a.getPosition().x][a.getPosition().y] = b;
		_Blocks[b.getPosition().x][b.getPosition().y] = a;
		
		int tempX = a.getPosition().x;
		int tempY = a.getPosition().y;
		a.setPosition(b.getPosition());
		b.setPosition(tempX, tempY);
	}
	
	@Override
	public void moveBlockLeft(int x, int y) {
		if (x > 0) swapBlocks(_Blocks[x][y], _Blocks[x-1][y]);
	}

	@Override
	public void moveBlockRight(int x, int y) {
		if (x < (_Width-1)) swapBlocks(_Blocks[x][y], _Blocks[x+1][y]);
	}

	@Override
	public void moveBlockUp(int x, int y) {
		if (y > 0) swapBlocks(_Blocks[x][y], _Blocks[x][y-1]);
	}

	@Override
	public void moveBlockDown(int x, int y) {
		if (y < (_Height-1)) swapBlocks(_Blocks[x][y], _Blocks[x][y+1]);
	}
	
	private OnNotifyEventListener _OnBlockClick = new OnNotifyEventListener() {
		@Override
		public void onNotify(Object sender) {
			Block block = (Block) sender; 
			_Board.getBlockControl().slide(block.getPosition());
		}
	};
	
}
